package com.zanxiang.game.module.manage.pojo.params;

import com.zanxiang.game.module.manage.enums.KfWebSocketMsgEnum;
import lombok.Data;

/**
 * @author : lingfeng
 * @time : 2024-02-27
 * @description : WebSocket客服消息通信参数
 */
@Data
public class KfWebSocketMsgParam {

    /**
     * webSocket消息类型
     */
    private KfWebSocketMsgEnum webSocketMsgType;

    /**
     * 客服登录token, 消息类型为首次连接(WEBSOCKET_MSG_CONNECT)时, 必须携带, 否则无法建立连接
     */
    private String token;

    /**
     * 分页信息, 分页获取数据的时候, 必须携带
     */
    private PageBean page;

    /**
     * 玩家id, 玩家接入的时候必须传
     */
    private String openId;

    /**
     * 房间id, 获取房间历史消息必传
     */
    private Long roomId;

    /**
     * 消息内容, 客服发送消息的时候必传
     */
    private MsgContentBean msgContent;

    @Data
    public static class MsgContentBean {

        /**
         * 消息类型, 文本传 : text, 图片传 : image
         */
        private String msgType;

        /**
         * 文本内容
         */
        private String text;

        /**
         * 图片地址(oss下载地址)
         */
        private String picUrl;
    }

    @Data
    public static class PageBean {

        /**
         * 当前页数
         */
        private Long pageNum;

        /**
         * 单页数量
         */
        private Long pageSize;
    }
}
