package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.module.manage.enums.KfRoomMsgTypeEnum;
import com.zanxiang.game.module.manage.pojo.dto.KfWebSocketMsgDTO;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.*;
import com.zanxiang.game.module.mybatis.mapper.KfRoomMapper;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2024-02-23
 * @description : 客服聊天房间
 */
@Slf4j
@Service
public class KfRoomServiceImpl extends ServiceImpl<KfRoomMapper, KfRoom> implements IKfRoomService {

    @Autowired
    private IGameAuthService gameAuthService;

    @Autowired
    private IGameAppletService gameAppletService;

    @Autowired
    private IGameUserRoleService gameUserRoleService;

    @Autowired
    private IKfRoomMsgService kfRoomMsgService;

    @Override
    public List<KfWebSocketMsgDTO.GameBean> getKfGameList() {
        List<GameAuth> gameAuthList = gameAuthService.list(new LambdaQueryWrapper<GameAuth>()
                .eq(!SecurityUtil.isAdmin(), GameAuth::getUserId, SecurityUtil.getUserId()));
        if (!SecurityUtil.isAdmin() && CollectionUtils.isEmpty(gameAuthList)) {
            return Collections.emptyList();
        }
        return gameAppletService.list(new LambdaQueryWrapper<GameApplet>()
                .in(!SecurityUtil.isAdmin(), GameApplet::getGameId,
                        gameAuthList.stream().map(GameAuth::getGameId).collect(Collectors.toSet()))
                .eq(GameApplet::getType, 1)
        ).stream().map(this::transform).collect(Collectors.toList());
    }

    private KfWebSocketMsgDTO.GameBean transform(GameApplet gameApplet) {
        if (gameApplet == null) {
            return null;
        }
        return BeanUtil.copy(gameApplet, KfWebSocketMsgDTO.GameBean.class);
    }

    @Override
    public List<KfWebSocketMsgDTO.RoomBean> getOnlineRoomList() {
        return super.list(new LambdaQueryWrapper<KfRoom>()
                .eq(KfRoom::getKfUserId, SecurityUtil.getUserId())
                .eq(KfRoom::getOnline, Boolean.TRUE)
        ).stream().map(this::transform).collect(Collectors.toList());
    }

    private KfWebSocketMsgDTO.RoomBean transform(KfRoom kfRoom) {
        if (kfRoom == null) {
            return null;
        }
        KfWebSocketMsgDTO.RoomBean roomBean = BeanUtil.copy(kfRoom, KfWebSocketMsgDTO.RoomBean.class);
        //最近角色信息
        GameUserRole gameUserRole = null;
        if (kfRoom.getUserId() != null) {
            gameUserRole = gameUserRoleService.getLastGameUserRoleName(kfRoom.getUserId(), kfRoom.getGameId());
        }
        if (gameUserRole != null) {
            roomBean.setLastRoleName(gameUserRole.getRoleName());
            roomBean.setLastRoleId(gameUserRole.getRoleId());
        }
        //最后一条消息
        KfRoomMsg kfRoomMsg = kfRoomMsgService.getOne(new LambdaQueryWrapper<KfRoomMsg>()
                .eq(KfRoomMsg::getRoomId, kfRoom.getId())
                .orderByDesc(KfRoomMsg::getCreateTime)
                .last("limit 1"));
        if (kfRoomMsg != null) {
            if (Objects.equals(kfRoomMsg.getMsgType(), KfRoomMsgTypeEnum.KF_MSG_TYPE_TEXT.getValue())) {
                Map<String, String> textMap = JsonUtil.toMap(kfRoomMsg.getContent(), Map.class, String.class);
                roomBean.setLastMsg(textMap.get("text"));
            }
            if (Objects.equals(kfRoomMsg.getMsgType(), KfRoomMsgTypeEnum.KF_MSG_TYPE_LINK.getValue())) {
                roomBean.setLastMsg("充值消息, 请在聊天中查看");
            }
            if (Objects.equals(kfRoomMsg.getMsgType(), KfRoomMsgTypeEnum.KF_MSG_TYPE_IMAGE.getValue())) {
                roomBean.setLastMsg("图片消息, 请在聊天中查看");
            }
            roomBean.setLastMsgTime(kfRoomMsg.getCreateTime());
        }
        return roomBean;
    }
}
